import { getShare, getQrcode, imageTransfer } from '../../api/base.js'
import { wxml, style } from './share.js'
const app = getApp()

Component({
  options: {
    styleIsolation: 'shared'
  },
  /**
   * 组件的属性列表
   */
  properties: {
    semesterId: { type: String, value: '' },
    courseId: { type: String, value: '' },
    chapterId: { type: String, value: '' },
    isNeed: { type: Boolean, value: false }
  },

  /**
   * 组件的初始数据
   */
  data: {
    show: false,
    hasControls: false,
    isPreivew: false,
    user: {},
    share: { lecturer: {} },
    qrcode: '',
    bgList: [
      'https://webapp-pub.ezijing.com/weapp/share/bg/02.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/03.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/04.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/05.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/06.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/07.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/08.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/09.jpg',
      'https://webapp-pub.ezijing.com/weapp/share/bg/01.jpg'
    ],
    textList: [],
    avatarList: [],
    colorList: [
      '#0E386B',
      '#d78c4a',
      '#f44336',
      '#e91e63',
      '#673ab7',
      '#3f51b5',
      '#2196f3',
      '#00bcd4',
      '#009688',
      '#4caf50',
      '#8bc34a',
      '#cddc39',
      '#ffeb3b',
      '#ffc107',
      '#ff9800',
      '#ff5722',
      '#795548',
      '#9e9e9e',
      '#607d8b'
    ],
    controls: {
      active: '0',
      bg: 'https://webapp-pub.ezijing.com/weapp/share/bg/02.jpg',
      text: '',
      avatar: '',
      color: '#0E386B'
    },
    controlsTabs: [
      { index: '0', text: '背景' },
      { index: '1', text: '金句' },
      { index: '3', text: '颜色' },
      { index: '2', text: '头像' }
    ],
    wxUserProfile: {},
    numLength: 1,
    customText: '',
    customTextVisible: false,
    keyboardHeight: 0
  },
  lifetimes: {
    attached() {
      // 加载自定义字体
      // wx.loadFontFace({ family: 'my', source: 'https://webapp-pub.ezijing.com/weapp/share/my.ttf' })
    }
  },
  /**
   * 组件的方法列表
   */
  methods: {
    init() {
      // 触发分享海报上报
      wx.reportMonitor('1', 1)
      this.getShare().then(res => {
        if (!this.data.isNeed && !res.data.need_share) {
          return
        }
        app.getUser().then(res => {
          res.realname = res.realname || res.nickname
          // 系统头像
          const avatarList = []
          avatarList.push({ name: '系统', url: res.avatar || 'https://webapp-pub.ezijing.com/weapp/share/default.jpg' })
          // 微信头像
          const localUserInfo = wx.getStorageSync('userInfo')
          let wxUserProfile = localUserInfo ? JSON.parse(localUserInfo) : {}
          if (wxUserProfile.avatarUrl) {
            avatarList.push({ name: '微信', url: wxUserProfile.avatarUrl })
          }
          const [firstAvatar = {}] = avatarList
          this.setData({ user: res, wxUserProfile, avatarList, show: true, 'controls.avatar': firstAvatar.url })
          this.getQrcode()
        })
        // 显示分享海报上报
        wx.reportMonitor('2', 1)
      })
    },
    // 获取营销数据
    getShare() {
      const params = {
        semester_id: this.data.semesterId,
        course_id: this.data.courseId,
        chapter_id: this.data.chapterId
      }
      return getShare(params).then(res => {
        const data = res.data
        const [lecturer = { lecturer_name: '' }] = data.lecturers
        lecturer.lecturer_name = lecturer.lecturer_name ? lecturer.lecturer_name.replace(/(\(|（)(.+)(\)|）)/, '') : ''
        data.course_name = data.course_name ? data.course_name.replace(/(\(|（)(.+)(\)|）)/, '') : ''
        data.course_name = `《${data.course_name}》课程`
        data.chapter_name = data.chapter_name ? data.chapter_name.replace(/(\(|（)(.+)(\)|）)/, '') : ''
        data.chapter_name = data.chapter_name ? data.chapter_name.replace(/^第.*章\s/, '') : ''
        data.chapter_name = data.chapter_name ? `“${data.chapter_name}”专题` : ''

        const numLength = data.used_day.toString().length
        // 金句设置
        const textList = data.sentences.map(item => {
          return { text: item }
        })
        if (textList.length) {
          this.setData({ textList, 'controls.text': textList[0].text })
        }
        this.setData({ share: Object.assign(data, { lecturer }), numLength })
        return res
      })
    },
    // 生成二维码
    getQrcode() {
      const user = this.data.user
      const params = {
        url: `https://pages.ezijing.com/prp/mobile202108260501.html?channel_num=93530&user_id=${user.id}&user_name=${user.realname}`
      }
      getQrcode(params).then(res => {
        this.setData({ qrcode: res.url })
      })
    },
    getUserProfile() {
      // 获取微信头像
      wx.getUserProfile({ desc: '使用头像生成海报' })
        .then(res => {
          const userInfo = res.userInfo
          imageTransfer({ source: userInfo.avatarUrl }).then(res => {
            userInfo.avatarUrl = res
            this.data.avatarList.push({ name: '微信', url: userInfo.avatarUrl })
            this.setData({ wxUserProfile: userInfo, avatarList: this.data.avatarList })
            wx.setStorageSync('userInfo', JSON.stringify(userInfo))
          })
        })
        .catch(() => {
          wx.showToast({ title: '获取微信头像失败', icon: 'none' })
        })
    },
    renderToCanvas() {
      wx.showLoading({ title: '海报生成中' })
      this.setData({ isPreivew: true }, () => {
        this.widget = this.selectComponent('.widget')
        setTimeout(() => {
          this.widget
            .renderToCanvas({ wxml: wxml(this.data), style: style(this.data) })
            .then(res => {
              wx.hideLoading()
            })
            .catch(error => {
              console.log(error)
              wx.hideLoading()
              wx.showToast({ title: '海报生成失败', icon: 'none' })
            })
        }, 500)
      })
      // 生成分享海报上报
      wx.reportMonitor('3', 1)
    },
    // 导出图片
    extraImage() {
      this.widget.canvasToTempFilePath().then(res => {
        wx.saveImageToPhotosAlbum({
          filePath: res.tempFilePath
        })
          .then(res => {
            wx.showToast({ title: '保存成功', icon: 'none' })
          })
          .catch(error => {
            wx.showToast({ title: '保存失败，请检查相册权限', icon: 'none' })
          })
      })
      // 保存分享海报上报
      wx.reportMonitor('0', 1)
    },
    // 关闭分享
    onClose() {
      this.setData({ show: false })
    },
    // 背景|金句选项卡切换
    onTabChange(e) {
      const { index } = e.currentTarget.dataset
      this.setData({ 'controls.active': index })
    },
    // 选择背景
    onSelectImage(e) {
      const { item } = e.currentTarget.dataset
      this.setData({ 'controls.bg': item })
    },
    // 选择金句
    onSelectText(e) {
      const { item } = e.currentTarget.dataset
      this.setData({ 'controls.text': item.text })
    },
    // 选择头像
    onSelectAvatar(e) {
      const { item } = e.currentTarget.dataset
      this.setData({ 'controls.avatar': item.url })
    },
    // 选择颜色
    onSelectColor(e) {
      const { item } = e.currentTarget.dataset
      this.setData({ 'controls.color': item })
    },
    // 控制栏显示隐藏
    toggleControls() {
      this.setData({ hasControls: !this.data.hasControls })
    },
    // 关闭预览
    onClosePreview() {
      this.setData({ isPreivew: false })
    },
    // 显示自定义金句
    showCustomText() {
      this.setData({ customTextVisible: true })
    },
    // 使用自定义金句
    onCustomTextConfirm() {
      if (this.data.customText) {
        this.data.textList.unshift({ text: this.data.customText, isCustom: true })
        this.setData({
          'controls.text': this.data.customText,
          textList: this.data.textList,
          customTextVisible: false,
          customText: ''
        })
      }
    },
    // 失去焦点
    onCustomTextBlur() {
      this.setData({ customTextVisible: false })
    },
    // 计算键盘高度
    onKeyboardheightchange(e) {
      this.setData({ keyboardHeight: e.detail.height })
    },
    // 删除自定义金句
    onRemoveCustomText(e) {
      const { index } = e.currentTarget.dataset
      this.data.textList.splice(index, 1)
      const found = this.data.textList.find(item => item.text === this.data.controls.text)
      this.setData({ textList: this.data.textList, 'controls.text': found ? this.data.controls.text : this.data.textList[0].text })
    }
  }
})
